"use strict";
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const chalk = require("chalk");
const cli_utils_1 = require("@ionic/cli-utils");
const upload_1 = require("../../lib/upload");
let PackageBuildCommand = class PackageBuildCommand extends cli_utils_1.Command {
    preRun(inputs, options) {
        return __awaiter(this, void 0, void 0, function* () {
            const token = yield this.env.session.getAppUserToken();
            const pkg = new cli_utils_1.PackageClient(token, this.env.client);
            const sec = new cli_utils_1.SecurityClient(token, this.env.client);
            if (!inputs[0]) {
                const platform = yield this.env.prompt({
                    type: 'list',
                    name: 'platform',
                    message: 'What platform would you like to target:',
                    choices: ['ios', 'android'],
                });
                inputs[0] = platform;
            }
            if (!options['profile'] && (inputs[0] === 'ios' || (inputs[0] === 'android' && options['release']))) {
                this.env.tasks.next(`Build requires security profile, but ${chalk.green('--profile')} was not provided. Looking up your profiles`);
                const allProfiles = yield sec.getProfiles({});
                this.env.tasks.end();
                const desiredProfileType = options['release'] ? 'production' : 'development';
                const profiles = allProfiles.filter(p => p.type === desiredProfileType);
                if (profiles.length === 0) {
                    this.env.log.error(`Sorry--a valid ${chalk.bold(desiredProfileType)} security profile is required for ${pkg.formatPlatform(inputs[0])} ${options['release'] ? 'release' : 'debug'} builds.`);
                    return 1;
                }
                if (profiles.length === 1) {
                    this.env.log.warn(`Attempting to use ${chalk.bold(profiles[0].tag)} (${chalk.bold(profiles[0].name)}), as it is your only ${chalk.bold(desiredProfileType)} security profile.`);
                    options['profile'] = profiles[0].tag;
                }
                else {
                    const profile = yield this.env.prompt({
                        type: 'list',
                        name: 'profile',
                        message: 'Please choose a security profile to use with this build',
                        choices: profiles.map(p => ({
                            name: p.name,
                            short: p.name,
                            value: p.tag,
                        })),
                    });
                    options['profile'] = profile;
                }
            }
        });
    }
    run(inputs, options) {
        return __awaiter(this, void 0, void 0, function* () {
            let [platform] = inputs;
            let { prod, release, profile, note } = options;
            if (typeof note !== 'string') {
                note = 'Ionic Package Upload';
            }
            const project = yield this.env.project.load();
            const token = yield this.env.session.getAppUserToken();
            const deploy = new cli_utils_1.DeployClient(token, this.env.client);
            const sec = new cli_utils_1.SecurityClient(token, this.env.client);
            const pkg = new cli_utils_1.PackageClient(token, this.env.client);
            if (typeof profile === 'string') {
                this.env.tasks.next(`Retrieving security profile ${chalk.bold(profile)}`);
                const p = yield sec.getProfile(profile.toLowerCase());
                this.env.tasks.end();
                if (!p.credentials[platform]) {
                    this.env.log.error(`Profile ${chalk.bold(p.tag)} (${chalk.bold(p.name)}) was found, but didn't have credentials for ${pkg.formatPlatform(platform)}.`);
                    return 1;
                }
                if (release && p.type !== 'production') {
                    this.env.log.error(`Profile ${chalk.bold(p.tag)} (${chalk.bold(p.name)}) is a ${chalk.bold(p.type)} profile, which won't work for release builds.\n` +
                        `Please use a production security profile.`);
                    return 1;
                }
            }
            if (project.type === 'ionic-angular' && release && !prod) {
                this.env.log.warn(`We recommend using ${chalk.green('--prod')} for production builds when using ${chalk.green('--release')}.`);
            }
            this.env.tasks.end();
            yield this.env.hooks.fire('command:build', {
                cmd: this,
                env: this.env,
                inputs,
                options: cli_utils_1.filterOptionsByIntent(this.metadata, options, 'app-scripts'),
            });
            const snapshotRequest = yield upload_1.upload(this.env, { note });
            this.env.tasks.next('Requesting project upload');
            const uploadTask = this.env.tasks.next('Uploading project');
            const proj = yield pkg.requestProjectUpload();
            const zip = cli_utils_1.createArchive('zip');
            zip.file('package.json');
            zip.file('config.xml');
            zip.directory('resources');
            zip.finalize();
            yield pkg.uploadProject(proj, zip, { progress: (loaded, total) => {
                    uploadTask.progress(loaded, total);
                } });
            this.env.tasks.next('Queuing build');
            const snapshot = yield deploy.getSnapshot(snapshotRequest.uuid, {});
            const build = yield pkg.queueBuild({
                platform,
                mode: release ? 'release' : 'debug',
                zipUrl: snapshot.url,
                projectId: proj.id,
                profileTag: typeof profile === 'string' ? profile : undefined,
            });
            this.env.tasks.end();
            this.env.log.ok(`Build ${build.id} has been submitted!`);
        });
    }
};
PackageBuildCommand = __decorate([
    cli_utils_1.CommandMetadata({
        name: 'build',
        type: 'project',
        description: 'Start a package build',
        inputs: [
            {
                name: 'platform',
                description: `The platform to target: ${chalk.green('ios')}, ${chalk.green('android')}`,
                validators: [cli_utils_1.contains(['ios', 'android'], {})],
            },
        ],
        options: [
            {
                name: 'prod',
                description: 'Mark as a production build',
                type: Boolean,
                intent: 'app-scripts',
            },
            {
                name: 'release',
                description: 'Mark as a release build',
                type: Boolean,
            },
            {
                name: 'profile',
                description: 'The security profile to use with this build',
                type: String,
                aliases: ['p'],
            },
            {
                name: 'note',
                description: 'Give the package snapshot a note',
            },
        ],
        exampleCommands: ['android', 'ios --profile dev', 'android --profile prod --release --prod'],
    })
], PackageBuildCommand);
exports.PackageBuildCommand = PackageBuildCommand;
