"use strict";
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const fs = require("fs");
const path = require("path");
const chalk = require("chalk");
const cli_utils_1 = require("@ionic/cli-utils");
let PackageDownloadCommand = class PackageDownloadCommand extends cli_utils_1.Command {
    run(inputs, options) {
        return __awaiter(this, void 0, void 0, function* () {
            let [id] = inputs;
            let { destination } = options;
            const destDir = path.resolve(destination || this.env.project.directory);
            let buildId = isNaN(Number(id)) ? undefined : Number(id);
            const token = yield this.env.session.getAppUserToken();
            const pkg = new cli_utils_1.PackageClient(token, this.env.client);
            if (buildId) {
                this.env.tasks.next(`Retrieving information about build ${chalk.bold(String(buildId))}`);
            }
            else {
                this.env.tasks.next('Retrieving latest build information');
                const latestBuilds = yield pkg.getBuilds({ pageSize: 1 });
                if (latestBuilds.length === 0) {
                    this.env.tasks.end();
                    return this.env.log.warn(`You don't have any builds yet! Run ${chalk.green('ionic package build --help')} to learn how.`);
                }
                buildId = latestBuilds[0].id;
            }
            const build = yield pkg.getBuild(buildId, {});
            const buildFilename = path.join(destDir, pkg.formatFilename(build));
            this.env.tasks.end();
            const ws = fs.createWriteStream(buildFilename);
            const downloadTask = this.env.tasks.next(`Downloading build ${chalk.bold(String(build.id))}`);
            yield pkg.downloadBuild(build, ws, { progress: (loaded, total) => {
                    downloadTask.progress(loaded, total);
                } });
            this.env.tasks.end();
            this.env.log.ok(`Build ${chalk.bold(String(build.id))} downloaded as ${chalk.bold(cli_utils_1.prettyPath(buildFilename))}.`);
        });
    }
};
PackageDownloadCommand = __decorate([
    cli_utils_1.CommandMetadata({
        name: 'download',
        type: 'project',
        description: 'Download your packaged app',
        inputs: [
            {
                name: 'id',
                description: 'The build ID to download. Defaults to the latest build',
                required: false,
            },
        ],
        options: [
            {
                name: 'destination',
                description: 'The download destination directory',
                aliases: ['d'],
            },
        ],
        exampleCommands: ['', '15', '-d my_builds'],
    })
], PackageDownloadCommand);
exports.PackageDownloadCommand = PackageDownloadCommand;
